//----------------------------------------------------------------------------
// * Node Default Initilization {node_init_default}
//----------------------------------------------------------------------------
//	Initializes a parentless and childless node with ID = 0 and an idle
//	state variable.
//----------------------------------------------------------------------------
Node::Node(){
	data_id = 0;
	data_parent = 0;
	state = Idle;
	actions_counter = 0;
	causality = 0;
}

//----------------------------------------------------------------------------
// * Node Initilization {node_init_id}
//----------------------------------------------------------------------------
//	Initializes a parentless and childless node with ID = node_id and an
//	idle state variable.
//----------------------------------------------------------------------------
Node::Node(int node_id){
	data_id = node_id;
	data_parent = 0;
	state = Idle;
	actions_counter = 0;
	causality = 0;
}

//----------------------------------------------------------------------------
// * Node Default Initilization {node_init_state}
//----------------------------------------------------------------------------
//	Initializes a parentless and childless node with ID = node_id and
//	state variable set to node_state.
//----------------------------------------------------------------------------
Node::Node(int node_id, states node_state){
	data_id = node_id;
	data_parent = 0;
	state = node_state;
	actions_counter = 0;
	causality = 0;
}

//----------------------------------------------------------------------------
// * Node Copy Constructor {node_copy_constructor}
//----------------------------------------------------------------------------
//	Initializes the contents of this node as the copy of another node. All
//	links and ancestral data are abandoned to avoid cross-tree pointers.
//----------------------------------------------------------------------------
Node::Node(const Node &copy){
	data_id = copy.id();
	data_parent = 0;
	state = copy.state;
	actions_counter = copy.actions_performed();
	causality = copy.get_causality();

	std::vector<const Action*>::const_iterator copy_action;
	const Action* action;

	for(copy_action = copy.actions.begin(); copy_action < copy.actions.end(); copy_action++){
		action = new Action(this, (*copy_action)->id());
		actions.push_back(action);
	}
}

//----------------------------------------------------------------------------
// * Node Destructor {node_destructor}
//----------------------------------------------------------------------------
//	Handles the termination of a node, deleting all actions assigned to
//	the node.
//----------------------------------------------------------------------------
Node::~Node(){
	std::vector<const Action*>::iterator action;

	for(action = actions.begin(); action < actions.end(); action++){
		delete (*action);
	}
}

//----------------------------------------------------------------------------
// * Node Is Linked? {node_linked}
//----------------------------------------------------------------------------
//	Returns true if the node is joined to any other node. Returns false
//	otherwise.
//----------------------------------------------------------------------------
bool Node::linked() const{
	return !links.empty();
}

//----------------------------------------------------------------------------
// * Node Is Neighbor? {node_neighboring}
//----------------------------------------------------------------------------
//	Returns true if node is a neighbor of this node. Returns false if it
//	is not.
//----------------------------------------------------------------------------
bool Node::neighboring(const Node* node) const{
	bool neighbor = false;

	std::vector<const Node*>::const_iterator link;

	for(link = links_begin(); link < links_end(); link++){
		if(*link == node){
			neighbor = true;
			break;
		}
	}

	return neighbor;
}

//----------------------------------------------------------------------------
// * Is Root? {node_root}
//---------------------------------------------------------------------------- 
// 	Returns true if the node has no parent, but has children. Returns
//	false otherwise.
//---------------------------------------------------------------------------- 
bool Node::root() const{
	return (data_parent == 0 && !children.empty());
}

//----------------------------------------------------------------------------
// * Is Intermediate? {node_intermediate}
//---------------------------------------------------------------------------- 
//	Returns true if the node has both a parent and  children. Returns
//	false otherwise.
//---------------------------------------------------------------------------- 
bool Node::intermediate() const{
	return(!children.empty() && data_parent > 0);
}

//----------------------------------------------------------------------------
// * Is Leaf? {node_leaf}
//---------------------------------------------------------------------------- 
//	Returns true if the node has a parent, but no children. Returns false
//	otherwise.
//---------------------------------------------------------------------------- 
bool Node::leaf() const{
	return(children.empty() && data_parent > 0);
}

//----------------------------------------------------------------------------
// * Rooted? {node_rooted}
//---------------------------------------------------------------------------- 
// 	Returns true if the node is part of a rooted tree, and has either a
//	parent, or has children. Returns false otherwise.
//---------------------------------------------------------------------------- 
bool Node::rooted() const{
	return(!children.empty() || data_parent > 0);
}

//----------------------------------------------------------------------------
// * Get Node ID? {node_id}
//---------------------------------------------------------------------------- 
//	Returns the node's id
//---------------------------------------------------------------------------- 
int Node::id() const{
	return data_id;
}

//---------------------------------------------------------------------------- 
// * Requesting? {node_requesting}
//---------------------------------------------------------------------------- 
//	Returns whether or not the node's state is Requesting.
//---------------------------------------------------------------------------- 
bool Node::requesting() const{
	return(state == Requesting);
}

//---------------------------------------------------------------------------- 
// * Idle? {node_idle}
//---------------------------------------------------------------------------- 
//	Returns whether or not the node's state is Idle.
//---------------------------------------------------------------------------- 
bool Node::idle() const{
	return(state == Idle);
}

//---------------------------------------------------------------------------- 
// * Replying? {node_replying}
//---------------------------------------------------------------------------- 
//	Returns whether or not the node's state is Replying.
//---------------------------------------------------------------------------- 
bool Node::replying() const{
	return(state == Replying);
}

//---------------------------------------------------------------------------- 
// * Get Node State {node_get_state}
//---------------------------------------------------------------------------- 
//	Returns the node's state variable.
//----------------------------------------------------------------------------
states Node::get_state() const{
	return state;
}

//---------------------------------------------------------------------------- 
// * Get Node State String {node_state_str}
//---------------------------------------------------------------------------- 
//	Returns the node's state variable as a string for printing.
//----------------------------------------------------------------------------
std::string Node::state_str() const{
	if(requesting()) return "Requesting";
	if(idle()) return "Idle";
	if(replying()) return "Replying";
}

//---------------------------------------------------------------------------- 
// * Get Causality Number {node_get_causality}
//---------------------------------------------------------------------------- 
//	Returns the node's causality number
//---------------------------------------------------------------------------- 
int Node::get_causality() const{
	return causality;
}

//---------------------------------------------------------------------------- 
// * Get Performed Actions {node_actions_performed}
//---------------------------------------------------------------------------- 
//	Returns the number of actions that have been performed by this node.
//---------------------------------------------------------------------------- 
int Node::actions_performed() const{
	return actions_counter;
}

//---------------------------------------------------------------------------- 
// * Get Parent {node_parent}
//---------------------------------------------------------------------------- 
//	Returns a pointer to this node's parent node.
//---------------------------------------------------------------------------- 
Node* Node::parent(){
	return data_parent;
}

//---------------------------------------------------------------------------- 
// * Get Parent {node_parent_const}
//---------------------------------------------------------------------------- 
//	Returns a constant pointer to this node's parent node.
//---------------------------------------------------------------------------- 
const Node* Node::parent() const{
	return data_parent;
}

//---------------------------------------------------------------------------- 
// * Links Begin Iterator {node_links_begin}
//---------------------------------------------------------------------------- 
//	Returns an iterator pointing to a node pointer which refers to the
//	first neighboring node.
//---------------------------------------------------------------------------- 
std::vector<Node*>::iterator Node::links_begin(){
	return links.begin();
}

//---------------------------------------------------------------------------- 
// * Links Begin Iterator {node_links_begin_const}
//---------------------------------------------------------------------------- 
//	Returns a constant iterator pointing to a node pointer which refers to
//	the first neighboring node.
//---------------------------------------------------------------------------- 
std::vector<const Node*>::const_iterator Node::links_begin() const{
	return links_const.begin();
}

//---------------------------------------------------------------------------- 
// * Links End Iterator {node_links_end}
//---------------------------------------------------------------------------- 
//	Returns an iterator pointing to a node pointer which refers to the
//	end of the neighboring nodes vector.
//---------------------------------------------------------------------------- 
std::vector<Node*>::iterator Node::links_end(){
	return links.end();
}

//---------------------------------------------------------------------------- 
// * Links End Iterator {node_links_end_const}
//---------------------------------------------------------------------------- 
//	Returns a constant iterator pointing to a node pointer which refers to
//	the end of the neighboring nodes vector.
//---------------------------------------------------------------------------- 
std::vector<const Node*>::const_iterator Node::links_end() const{
	return links_const.end();
}

//---------------------------------------------------------------------------- 
// * Children Begin Iterator {node_children_begin}
//---------------------------------------------------------------------------- 
//	Returns an iterator pointing to a node pointer which refers to the
//	first child node.
//---------------------------------------------------------------------------- 
std::vector<Node*>::iterator Node::children_begin(){
	return children.begin();
}

//---------------------------------------------------------------------------- 
// * Children Begin Iterator {node_children_begin_const}
//---------------------------------------------------------------------------- 
//	Returns a constant iterator pointing to a node pointer which refers to 
//	the first child node.
//---------------------------------------------------------------------------- 
std::vector<const Node*>::const_iterator Node::children_begin() const{
	return children_const.begin();
}

//---------------------------------------------------------------------------- 
// * Children End Iterator {node_children_end}
//---------------------------------------------------------------------------- 
//	Returns an iterator pointing to a node pointer which refers to the end
//	of the children nodes vector.
//---------------------------------------------------------------------------- 
std::vector<Node*>::iterator Node::children_end(){
	return children.end();
}

//---------------------------------------------------------------------------- 
// * Children End Iterator {node_children_end_const}
//---------------------------------------------------------------------------- 
//	Returns a constant iterator pointing to a node pointer which refers to
//	the end of the children nodes vector.
//---------------------------------------------------------------------------- 
std::vector<const Node*>::const_iterator Node::children_end() const{
	return children_const.end();
}

//---------------------------------------------------------------------------- 
// * Add Neighbor {node_add_link}
//---------------------------------------------------------------------------- 
//	Adds a node pointer to the neighboring nodes of this node.
//---------------------------------------------------------------------------- 
void Node::add_link(Node* link){
	links.push_back(link);
	links_const.push_back(link);
}

//---------------------------------------------------------------------------- 
// * Add Child {node_add_child}
//---------------------------------------------------------------------------- 
//	Adds a node pointer as one of this node's children.
//---------------------------------------------------------------------------- 
void Node::add_child(Node* child){
	children.push_back(child);
	children_const.push_back(child);
}

//---------------------------------------------------------------------------- 
// * Set Parent {node_set_parent}
//---------------------------------------------------------------------------- 
//	Sets this node's parent node pointer to a new_parent.
//---------------------------------------------------------------------------- 
void Node::set_parent(Node* new_parent){
	data_parent = new_parent;
}

//---------------------------------------------------------------------------- 
// * Set State {node_set_state}
//---------------------------------------------------------------------------- 
//	Sets the state variable of this node to a new_state.
//---------------------------------------------------------------------------- 
void Node::set_state(states new_state){
	state = new_state;
}

//---------------------------------------------------------------------------- 
// * Perturb {node_perturb}
//---------------------------------------------------------------------------- 
//	Randomly sets the node to a new perturbed state regardless of any
//	effect this may have on program stability. Returns the original state
//	of the node before perturbation.
//---------------------------------------------------------------------------- 
states Node::perturb(){
	states original_state = state;

	std::vector<states> available;

	available.push_back(Idle);

	if(root() || intermediate()) available.push_back(Requesting);
	if(leaf() || intermediate()) available.push_back(Replying);

	state = available[rand() % available.size()];

	return original_state;
}

//---------------------------------------------------------------------------- 
// * Set Causality Number {node_set_causality}
//---------------------------------------------------------------------------- 
//	Sets the causality number of this node to value.
//---------------------------------------------------------------------------- 
void Node::set_causality(int value){
	causality = value;
}


//---------------------------------------------------------------------------- 
// * Reset Actions Performed Counter {node_reset_actions}
//---------------------------------------------------------------------------- 
//	Resets the counter for actions performed to zero.
//---------------------------------------------------------------------------- 
void Node::reset_actions(){
	actions_counter = 0;
}

//---------------------------------------------------------------------------- 
// * Disown Children {node_disown}
//---------------------------------------------------------------------------- 
//	Clears the children vectors leaving the node childless.
//---------------------------------------------------------------------------- 
void Node::disown(){
	children.clear();
	children_const.clear();
}

//---------------------------------------------------------------------------- 
// * Define Actions {define_actions}
//---------------------------------------------------------------------------- 
//	Defines the actions available to this node.
//---------------------------------------------------------------------------- 
void Node::define_actions(){
	actions.clear();

	if(rooted()){
		if(root()){
			for(unsigned short int id = 0; id < 2; id++){
				Action *action;
				action = new Action(this, id);
				actions.push_back(action);
			}
		}
		else if(intermediate()){
			for(unsigned short int id = 2; id < 5; id++){
				Action *action;
				action = new Action(this, id);
				actions.push_back(action);
			}
		}
		else if(leaf()){
			for(unsigned short int id = 5; id < 7; id++){
				Action *action;
				action = new Action(this, id);
				actions.push_back(action);
			}
		}
	}
}

//---------------------------------------------------------------------------- 
// * Update Causality Number {node_update_causality}
//---------------------------------------------------------------------------- 
//	Sets the causality number of this node and its neighbors to 1 + the
//	maximum causality number between them.
//---------------------------------------------------------------------------- 
void Node::update_causality(){
	std::vector<Node*>::iterator node;
	
	int maximum = causality;

	for(node = links_begin(); node < links_end(); node++){
		if((*node)->get_causality() > maximum){
			maximum = (*node)->get_causality();
		}
	}

	causality = maximum + 1;

	for(node = links_begin(); node < links_end(); node++){
		(*node)->set_causality(maximum + 1);
	}
}

//---------------------------------------------------------------------------- 
// * Get Enabled Action {node_enabled}
//---------------------------------------------------------------------------- 
//	Returns a constant pointer to the action which is currently enabled
//	for this process.
//---------------------------------------------------------------------------- 
const Action* Node::enabled() const{
	const Action *enabled_action = 0;
	std::vector<const Action*>::const_iterator action;

	for(action = actions.begin(); action < actions.end(); action++){
		if((*action)->guard()){
			enabled_action = *action;
			break;
		}
	}

	return enabled_action;
}

//---------------------------------------------------------------------------- 
// * Execute Enabled Action {node_execute}
//---------------------------------------------------------------------------- 
//	Executes an action which is enabled for this node.
//---------------------------------------------------------------------------- 
const Action* Node::execute(){
	if(enabled() > 0){
		enabled()->execute();
		actions_counter++;
	}

	return enabled();
}

//---------------------------------------------------------------------------- 
// * Node Assignment Operator {node_assignment_operator}
//---------------------------------------------------------------------------- 
//	Assigns the contents of another node to this node. All links and
//	ancestral data are abandoned to avoid cross-tree pointers.
//---------------------------------------------------------------------------- 
Node* Node::operator=(const Node &assignment){
	data_id = assignment.id();
	data_parent = 0;
	state = assignment.state;
	actions_counter = assignment.actions_performed();
	causality = assignment.get_causality();

	std::vector<const Action*>::const_iterator assignment_action;
	const Action* action;

	actions.clear();

	for(assignment_action = assignment.actions.begin(); assignment_action < assignment.actions.end(); assignment_action++){
		action = new Action(this, (*assignment_action)->id());
		actions.push_back(action);
	}

	return this;
}

//---------------------------------------------------------------------------- 
// * Children Are State? {node_children_are}
//---------------------------------------------------------------------------- 
//	Returns true if all of this node's children's state variables are
//	equal to child_state. Returns false otherwise.
//---------------------------------------------------------------------------- 
bool Node::children_are(states child_state) const{
	bool result = true;
	std::vector<const Node*>::const_iterator child;

	for(child = children_begin(); child < children_end(); child++){
		if((*child)->state != child_state){
			result = false;
			break;
		}
	}

	return result;
}
