//----------------------------------------------------------------------------
// * Randomized Initialization
//----------------------------------------------------------------------------
//	Initializes a prufer sequence of length random integers.
//----------------------------------------------------------------------------
Prufer::Prufer(std::vector<int>::size_type length){
	data.resize(length);

	for(unsigned int i = 0; i < length; i++){
		data[i] = rand() % (length + 2);
	}
}

//----------------------------------------------------------------------------
// * Copy Constructor
//----------------------------------------------------------------------------
//	Copies the contents of a prufer sequence on initialization.
//----------------------------------------------------------------------------
Prufer::Prufer(const Prufer &copy){
	data = copy.data;
}

//----------------------------------------------------------------------------
// * Copy Constructor
//----------------------------------------------------------------------------
//	Constructs a prufer sequence from the contents of an integer vector.
//----------------------------------------------------------------------------
Prufer::Prufer(const std::vector<int> &sequence){
	data = sequence;
}

//----------------------------------------------------------------------------
// * Assignment Operator
//----------------------------------------------------------------------------
//	Assigns the contents of a prufer sequence to another.
//----------------------------------------------------------------------------
Prufer& Prufer::operator=(const Prufer &copy){
	data = copy.data;

	return *this;
}

//----------------------------------------------------------------------------
// * Assignment Operator
//----------------------------------------------------------------------------
//	Assigns the contents of an integer vector to a prufer sequence.
//----------------------------------------------------------------------------
Prufer& Prufer::operator=(const std::vector<int> &sequence){
	data = sequence;

	return *this;
}

//----------------------------------------------------------------------------
// * Begin Iterator {prufer_begin}
//----------------------------------------------------------------------------
//	Returns an integer vector iterator that points to the first number in
//	the sequence.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::begin(){
	return data.begin();
}

//----------------------------------------------------------------------------
// * Begin Iterator {prufer_begin_const}
//----------------------------------------------------------------------------
//	Returns an integer vector constant iterator that points to the first
//	number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::const_iterator Prufer::begin() const{
	return data.begin();
}

//----------------------------------------------------------------------------
// * End Iterator {prufer_end}
//----------------------------------------------------------------------------
//	Returns an integer vector iterator that points to the position after
//	the last number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::end(){
	return data.end();
}

//----------------------------------------------------------------------------
// * End Iterator {prufer_end_const}
//----------------------------------------------------------------------------
//	Returns an integer vector constant iterator that points to the
//	poisition after the last number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::const_iterator Prufer::end() const{
	return data.end();
}

//----------------------------------------------------------------------------
// * Begin Reverse Iterator {prufer_rbegin}
//----------------------------------------------------------------------------
//	Returns an integer vector constant reverse iterator that points to the
//	last number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::reverse_iterator Prufer::rbegin(){
	return data.rbegin();
}

//----------------------------------------------------------------------------
// * Begin Reverse Iterator {prufer_rbegin}
//----------------------------------------------------------------------------
//	Returns an integer vector reverse iterator that points to the last
//	number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::const_reverse_iterator Prufer::rbegin() const{
	return data.rbegin();
}

//----------------------------------------------------------------------------
// * End Reverse Iterator {prufer_rend}
//----------------------------------------------------------------------------
//	Returns an integer vector reverse iterator that points to the position
//	just before the first number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::reverse_iterator Prufer::rend(){
	return data.rend();
}

//----------------------------------------------------------------------------
// * End Reverse Iterator {prufer_rend_const}
//----------------------------------------------------------------------------
//	Returns an integer vector constant reverse iterator that points to the
//	position just before the first number in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::const_reverse_iterator Prufer::rend() const{
	return data.rend();
}

//----------------------------------------------------------------------------
// * Is Included {prufer_include}
//----------------------------------------------------------------------------
//	Returns true if query is found in the sequence, and returns false
//	otherwise.
//----------------------------------------------------------------------------
bool Prufer::include(int query) const{
	bool found = false;

	std::vector<int>::const_iterator number;

	for(number = data.begin(); number < data.end(); number++){
		if(*number == query){
			found = true;
			break;
		}
	}

	return found;
}

//----------------------------------------------------------------------------
// * Is Empty {prufer_empty}
//----------------------------------------------------------------------------
//	Returns true if sequence is empty, and returns false otherwise
//----------------------------------------------------------------------------
bool Prufer::empty() const{
	return data.empty();
}

//----------------------------------------------------------------------------
// * Get Sequence Length {prufer_size}
//----------------------------------------------------------------------------
//	Returns the current number of integers in the prufer sequence.
//----------------------------------------------------------------------------
std::vector<int>::size_type Prufer::size() const{
	return data.size();
}

//----------------------------------------------------------------------------
// * Maximum Length {prufer_max_size}
//----------------------------------------------------------------------------
//	Returns the maximum number of integers allowed to be in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::size_type Prufer::max_size() const{
	return data.max_size();
}

//----------------------------------------------------------------------------
// *  Resize {prufer_resize}
//----------------------------------------------------------------------------
//	Resizes the length of the prufer sequence while resetting to a random
//	sequence.
//----------------------------------------------------------------------------
void Prufer::resize(std::vector<int>::size_type length){
	data.resize(length);

	for(unsigned int i = 0; i < length; i++){
		data[i] = rand() % (length + 2);
	}
}

//----------------------------------------------------------------------------
// * Get Data Capacity {prufer_capacity}
//----------------------------------------------------------------------------
//	Returns the current allocation capacity of the prufer sequence data
//	vector.
//----------------------------------------------------------------------------
std::vector<int>::size_type Prufer::capacity() const{
	return data.capacity();
}

//----------------------------------------------------------------------------
// * Reserve Allocation Space {prufer_reserve}
//----------------------------------------------------------------------------
//	Changes the allocation capacity of the sequence's data vector.
//----------------------------------------------------------------------------
void Prufer::reserve(std::vector<int>::size_type new_capacity){
	data.reserve(new_capacity);
}

//----------------------------------------------------------------------------
// * Access Operator {prufer_access_const}
//----------------------------------------------------------------------------
//	Returns a constant reference to the number in the sequence at
//	position index.
//----------------------------------------------------------------------------
int Prufer::operator[](std::vector<int>::size_type index) const{
	return data[index];
}

//----------------------------------------------------------------------------
// * Access Operator {prufer_access}
//----------------------------------------------------------------------------
//	Returns a reference to the number in the sequence at position index.
//----------------------------------------------------------------------------
int& Prufer::operator[](std::vector<int>::size_type index){
	return data[index];
}

//----------------------------------------------------------------------------
// * Element At {prufer_at_const}
//----------------------------------------------------------------------------
//	Returns a constant reference to the number in the sequence at
//	position index.
//----------------------------------------------------------------------------
int Prufer::at(std::vector<int>::size_type index) const{
	return data.at(index);
}

//----------------------------------------------------------------------------
// * Element At {prufer_access}
//----------------------------------------------------------------------------
//	Returns a reference to the number in the sequence at position index.
//----------------------------------------------------------------------------
int& Prufer::at(std::vector<int>::size_type index){
	return data.at(index);
}

//----------------------------------------------------------------------------
// * Find Number {prufer_find}
//----------------------------------------------------------------------------
//	Returns an integer vector iterator to the first found instance of
//	the number query. Returns an end iterator if there is no match.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::find(int query){
	std::vector<int>::iterator number;

	for(number = data.begin(); number < data.begin(); number++){
		if(*number = query) break;
	}

	return number;
}

//----------------------------------------------------------------------------
// * Find Number {prufer_find_const}
//----------------------------------------------------------------------------
//	Returns an integer vector constant iterator to the first found
//	instance of the number query. Returns an end iterator if there is no
//	match.
//----------------------------------------------------------------------------
std::vector<int>::const_iterator Prufer::find(int query) const{
	std::vector<int>::const_iterator number;

	for(number = data.begin(); number < data.begin(); number++){
		if(*number == query) break;
	}

	return number;
}

//----------------------------------------------------------------------------
// * Get Front Element {prufer_front_const}
//----------------------------------------------------------------------------
//	Returns the first number in the sequence.
//----------------------------------------------------------------------------
int Prufer::front() const{
	return data.front();
}

//----------------------------------------------------------------------------
// * Get Front Element {prufer_front}
//----------------------------------------------------------------------------
//	Returns the first number in the sequence.
//----------------------------------------------------------------------------
int& Prufer::front(){
	return data.front();
}

//----------------------------------------------------------------------------
// * Get Back Element {prufer_back_const}
//----------------------------------------------------------------------------
//	Returns the last number in the sequence.
//----------------------------------------------------------------------------
int Prufer::back() const{
	return data.back();
}

//----------------------------------------------------------------------------
// * Get Back Element {prufer_back}
//----------------------------------------------------------------------------
//	Returns the last number in the sequence.
//----------------------------------------------------------------------------
int& Prufer::back(){
	return data.back();
}

//----------------------------------------------------------------------------
// * Assign Elements {prufer_assign}
//----------------------------------------------------------------------------
//	Sets the sequence to the digits located between iterators first and
//	last.
//----------------------------------------------------------------------------
void Prufer::assign(std::vector<int>::iterator first, std::vector<int>::iterator last){
	data.assign(first, last);
}

//----------------------------------------------------------------------------
// * Assign Elements {prufer_assign_repeat}
//----------------------------------------------------------------------------
//	Sets the sequence to a repetition of number over length.
//----------------------------------------------------------------------------
void Prufer::assign(std::vector<int>::size_type length, int number){
	data.assign(length, number);
}

//----------------------------------------------------------------------------
// * Push Number {prufer_push_back}
//----------------------------------------------------------------------------
//	Pushes number to the end of the sequence.
//----------------------------------------------------------------------------
void Prufer::push_back(int number){
	data.push_back(number);
}

//----------------------------------------------------------------------------
// * Remove Last Number {prufer_pop_back}
//----------------------------------------------------------------------------
//	Erases the last number in the sequence.
//----------------------------------------------------------------------------
void Prufer::pop_back(){
	data.pop_back();
}

//----------------------------------------------------------------------------
// * Insert Number {prufer_insert}
//----------------------------------------------------------------------------
//	Inserts a number into the sequence at the location pointed to by the
//	position iterator.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::insert(std::vector<int>::iterator position, int number){
	return data.insert(position, number);
}

//----------------------------------------------------------------------------
// * Insert Number {prufer_insert_index}
//----------------------------------------------------------------------------
//	Inserts a number into the sequence at the given index
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::insert(std::vector<int>::size_type index, int number){
	unsigned int position = 0;
	std::vector<int>::iterator iter;

	for(iter = data.begin(); iter < data.end(); iter++, position++){
		if(position == index){
			iter = data.insert(iter, number);
			break;
		}
	}

	return iter;
}

//----------------------------------------------------------------------------
// * Insert Number {prufer_insert_range_repeat}
//----------------------------------------------------------------------------
//	Inserts a repetitive sequence of length numbers into the sequence at
//	the location pointed to by the position iterator.
//----------------------------------------------------------------------------
void Prufer::insert(std::vector<int>::iterator position, std::vector<int>::size_type length, int number){
	data.insert(position, length, number);
}

//----------------------------------------------------------------------------
// * Insert Number {prufer_insert_range}
//----------------------------------------------------------------------------
//	Inserts a range of numbers copied from the sequence between iterators
//	first and last into the sequence at the location pointed to by the
//	position iterator.
//----------------------------------------------------------------------------
void Prufer::insert(std::vector<int>::iterator position, std::vector<int>::iterator first, std::vector<int>::iterator last){
	data.insert(position, first, last);
}

//----------------------------------------------------------------------------
// * Remove Number {prufer_remove}
//----------------------------------------------------------------------------
//	Removes all instances of a number from the sequence. Returns true if
//	any matches were found and discards, and returns false if the number
//	was not found within the sequence.
//----------------------------------------------------------------------------
bool Prufer::remove(int query){
	bool discarded = false;
	std::vector<int>::iterator number;

	for(number = data.begin(); number < data.end(); number++){
		if(*number == query){
			discarded = true;
			number = data.erase(number);
		}
	}

	return discarded;
}

//----------------------------------------------------------------------------
// * Erase Number {prufer_erase}
//----------------------------------------------------------------------------
//	Erases the number in the sequence located at the position pointed to
//	by the position iterator. Returns an iterator pointing to the number
//	located after that which was erased. Returns the end iterator if the
//	erased number was the last in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::erase(std::vector<int>::iterator position){
	return data.erase(position);
}

//----------------------------------------------------------------------------
// * Erase Number {prufer_erase_index}
//----------------------------------------------------------------------------
//	Erases the number in the sequence located at the given index.  Returns
//	an iterator pointing to the number located after that which was
//	erased. Returns the end iterator if the erased number was the last
//	in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::erase(std::vector<int>::size_type index){
	unsigned int position = 0;
	std::vector<int>::iterator number;

	for(number = data.begin(); number < data.end(); number++, position++){
		if(position == index){
			number = data.erase(number);
			break;
		}
	}

	return number;
}

//----------------------------------------------------------------------------
// * Erase Number {prufer_erase_range}
//----------------------------------------------------------------------------
//	Erases the range of number in the sequence located between the first
//	and last iterators. Returns an iterator pointing to the number
//	located after the last number which was erased. Returns the end
//	iterator if the last number erased was the last in the sequence.
//----------------------------------------------------------------------------
std::vector<int>::iterator Prufer::erase(std::vector<int>::iterator first, std::vector<int>::iterator last){
	return data.erase(first, last);
}

//----------------------------------------------------------------------------
// * Swap {prufer_swap}
//----------------------------------------------------------------------------
//	Swaps the contents of one prufer sequence with another.
//----------------------------------------------------------------------------
void Prufer::swap(Prufer &other){
	data.swap(other.data);
}

//----------------------------------------------------------------------------
// * Clear {prufer_clear}
//----------------------------------------------------------------------------
//	Clears the entire contents of the prufer sequence.
//----------------------------------------------------------------------------
void Prufer::clear(){
	data.clear();
}

//----------------------------------------------------------------------------
// * Unique {prufer_unique}
//----------------------------------------------------------------------------
//	Removes all repetitive numbers in the sequence, leaving only the first
//	instance of each.
//----------------------------------------------------------------------------
void Prufer::unique(){
	Prufer counted;
	std::vector<int>::iterator number;

	for(number = data.begin(); number < data.end();){
		if(counted.include(*number)){
			number = data.erase(number);
		}
		else{
			counted.push_back(*number);
			number++;
		}
	}
}

//----------------------------------------------------------------------------
// * Get Allocator {prufer_get_allocator}
//----------------------------------------------------------------------------
//	Returns the allocator being used by the sequence data vector.
//----------------------------------------------------------------------------
std::vector<int>::allocator_type Prufer::get_allocator() const{
	return data.get_allocator();
}

//----------------------------------------------------------------------------
// * Output Operator Overload {prufer_output}
//----------------------------------------------------------------------------
//	Prints the sequence in the format: "{n1, n2, n3 ... }" to the given
//	output.
//----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream &out, const Prufer& sequence){
	out << '{';

	std::vector<int>::const_iterator number;

	for(number = sequence.begin(); number < sequence.end(); number++){
		out << *number;
		if(number < sequence.end() - 1) out << ", ";
	}

	out << '}';
}
